ReadWriteRawVec

Struct ReadWriteRawVec 

pub struct ReadWriteRawVec<I, T, S> { /* private fields */ }
Core implementation for raw storage vectors shared by BytesVec and ZeroCopyVec.

Parameterized by serialization strategy S to support different serialization approaches:

  • BytesStrategy: Explicit little-endian serialization (portable)
  • ZeroCopyStrategy: Native byte order via zerocopy (fast but not portable)

Provides holes (deleted indices) and updated values tracking for both vec types.

Implementations

Source

impl<I, T, S> ReadWriteRawVec<I, T, S>
where I: VecIndex, T: VecValue, S: RawStrategy<T>,

pub const SIZE_OF_T: usize

pub fn read_only_clone(&self) -> ReadOnlyRawVec<I, T, S>

pub fn forced_import_with( options: ImportOptions<'_>, format: Format, ) -> Result<Self>

Warning

This will DELETE all existing data on format/version errors. Use with caution.

pub fn import_with(options: ImportOptions<'_>, format: Format) -> Result<Self>

pub fn remove(self) -> Result<()>

pub fn holes(&self) -> &BTreeSet<usize>

pub fn prev_holes(&self) -> &BTreeSet<usize>

pub fn mut_holes(&mut self) -> &mut BTreeSet<usize>

pub fn updated(&self) -> &BTreeMap<usize, T>

pub fn mut_updated(&mut self) -> &mut BTreeMap<usize, T>

pub fn prev_updated(&self) -> &BTreeMap<usize, T>

pub fn pushed(&self) -> &[T]

pub fn mut_pushed(&mut self) -> &mut Vec<T>

pub fn push(&mut self, value: T)

pub fn reserve_pushed(&mut self, additional: usize)

pub fn create_reader(&self) -> Reader

pub fn reader(&self) -> VecReader<I, T, S>

pub fn index_to_name(&self) -> String

pub fn unchecked_read_at(&self, index: usize, reader: &Reader) -> T

pub fn read_at(&self, index: usize, reader: &Reader) -> Result<T>

pub fn read_at_once(&self, index: usize) -> Result<T>

pub fn read_once(&self, index: I) -> Result<T>

pub fn get_pushed_or_read( &self, index: I, reader: &VecReader<I, T, S>, ) -> Option<T>

pub fn get_pushed_or_read_at( &self, index: usize, reader: &VecReader<I, T, S>, ) -> Option<T>

pub fn get_any_or_read(&self, index: I, reader: &Reader) -> Result<Option<T>>

pub fn get_any_or_read_at( &self, index: usize, reader: &Reader, ) -> Result<Option<T>>

pub fn collect_holed(&self) -> Result<Vec<Option<T>>>

pub fn collect_holed_range( &self, from: usize, to: usize, ) -> Result<Vec<Option<T>>>

pub fn update(&mut self, index: I, value: T) -> Result<()>

pub fn update_at(&mut self, index: usize, value: T) -> Result<()>

pub fn delete(&mut self, index: I)

pub fn delete_at(&mut self, index: usize)

pub fn get_first_empty_index(&self) -> I

pub fn fill_first_hole_or_push(&mut self, value: T) -> Result<I>

pub fn take(&mut self, index: I, reader: &Reader) -> Result<Option<T>>

pub fn take_at(&mut self, index: usize, reader: &Reader) -> Result<Option<T>>

pub fn fold_stored_io<B, F: FnMut(B, T) -> B>( &self, from: usize, to: usize, init: B, f: F, ) -> B

pub fn fold_stored_mmap<B, F: FnMut(B, T) -> B>( &self, from: usize, to: usize, init: B, f: F, ) -> B