pub struct ZeroCopyStrategy<T>(/* private fields */);Expand description
Serialization strategy using zerocopy for native byte order access.
Uses direct memory mapping in native byte order - not portable across endianness.