pub trait AnyVecWithWriter: AnyReadableVec {
// Required method
fn create_writer(
&self,
from: Option<i64>,
to: Option<i64>,
) -> Box<dyn ValueWriter + '_>;
}Expand description
Type-erased trait for vecs that can produce a boxed row-by-row ValueWriter.
Required Methods
Sourcefn create_writer(
&self,
from: Option<i64>,
to: Option<i64>,
) -> Box<dyn ValueWriter + '_>
fn create_writer( &self, from: Option<i64>, to: Option<i64>, ) -> Box<dyn ValueWriter + '_>
Create a value writer that can be advanced row by row