AnyReadableVec

Trait AnyReadableVec 

pub trait AnyReadableVec: AnyVec {
    // Provided methods
    fn range_count(&self, from: Option<i64>, to: Option<i64>) -> usize { ... }
    fn range_weight(&self, from: Option<i64>, to: Option<i64>) -> usize { ... }
}
Type-erased trait for collectable vectors.

Provided Methods

fn range_count(&self, from: Option<i64>, to: Option<i64>) -> usize

Returns the number of items in the specified range.

fn range_weight(&self, from: Option<i64>, to: Option<i64>) -> usize

Returns the total size in bytes of items in the specified range.

Implementors

Source

impl<V> AnyReadableVec for V
where V: TypedVec + ReadableVec<V::I, V::T>,